@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2014 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,106 +22,89 @@
 
 #ifdef ENABLE_VISUALIZER
 
-#include "global.h"
-
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <cerrno>
 #include <cmath>
 #include <cstring>
 #include <fstream>
 #include <limits>
 #include <fcntl.h>
-#include <sys/time.h>
-#include <unistd.h>
+
+#include "global.h"
+#include "settings.h"
+#include "status.h"
+#include "statusbar.h"
+#include "title.h"
+#include "screen_switcher.h"
+#include "status.h"
 
 using Global::MainStartY;
 using Global::MainHeight;
 
-Visualizer *myVisualizer = new Visualizer;
+Visualizer *myVisualizer;
 
-const int Visualizer::WindowTimeout = 1000/25; /* 25 fps */
+namespace {
 
-void Visualizer::Init()
+const int fps = 25;
+
+}
+
+Visualizer::Visualizer()
+: Screen(NC::Window(0, MainStartY, COLS, MainHeight, "", Config.visualizer_color, NC::Border::None))
 {
-	w = new Window(0, MainStartY, COLS, MainHeight, "", Config.visualizer_color, brNone);
-	
 	ResetFD();
-	itsSamples = Config.visualizer_in_stereo ? 4096 : 2048;
+	m_samples = 44100/fps;
+	if (Config.visualizer_in_stereo)
+		m_samples *= 2;
 #	ifdef HAVE_FFTW3_H
-	itsFFTResults = itsSamples/2+1;
-	itsFreqsMagnitude = new unsigned[itsFFTResults];
-	itsInput = static_cast<double *>(fftw_malloc(sizeof(double)*itsSamples));
-	itsOutput = static_cast<fftw_complex *>(fftw_malloc(sizeof(fftw_complex)*itsFFTResults));
-	itsPlan = fftw_plan_dft_r2c_1d(itsSamples, itsInput, itsOutput, FFTW_ESTIMATE);
+	m_fftw_results = m_samples/2+1;
+	m_freq_magnitudes = new double[m_fftw_results];
+	m_fftw_input = static_cast<double *>(fftw_malloc(sizeof(double)*m_samples));
+	m_fftw_output = static_cast<fftw_complex *>(fftw_malloc(sizeof(fftw_complex)*m_fftw_results));
+	m_fftw_plan = fftw_plan_dft_r2c_1d(m_samples, m_fftw_input, m_fftw_output, FFTW_ESTIMATE);
 #	endif // HAVE_FFTW3_H
-	
-	FindOutputID();
-	
-	isInitialized = 1;
 }
 
-void Visualizer::SwitchTo()
+void Visualizer::switchTo()
 {
-	using Global::myScreen;
-	using Global::myLockedScreen;
-	
-	if (myScreen == this)
-		return;
-	
-	if (!isInitialized)
-		Init();
-	
-	if (myLockedScreen)
-		UpdateInactiveScreen(this);
-	
-	if (hasToBeResized || myLockedScreen)
-		Resize();
-	
-	if (myScreen != this && myScreen->isTabbable())
-		Global::myPrevScreen = myScreen;
-	myScreen = this;
-	w->Clear();
-	
+	SwitchTo::execute(this);
+	w.clear();
 	SetFD();
-	
-	itsTimer.tv_sec = 0;
-	itsTimer.tv_usec = 0;
-	
-	if (itsFifo >= 0)
-		Global::wFooter->SetTimeout(WindowTimeout);
-	Global::RedrawHeader = 1;
+	m_timer = boost::posix_time::from_time_t(0);
+	drawHeader();
 }
 
-void Visualizer::Resize()
+void Visualizer::resize()
 {
 	size_t x_offset, width;
-	GetWindowResizeParams(x_offset, width);
-	w->Resize(width, MainHeight);
-	w->MoveTo(x_offset, MainStartY);
+	getWindowResizeParams(x_offset, width);
+	w.resize(width, MainHeight);
+	w.moveTo(x_offset, MainStartY);
 	hasToBeResized = 0;
 }
 
-std::basic_string<my_char_t> Visualizer::Title()
+std::wstring Visualizer::title()
 {
-	return U("Music visualizer");
+	return L"Music visualizer";
 }
 
-void Visualizer::Update()
+void Visualizer::update()
 {
-	if (itsFifo < 0)
+	if (m_fifo < 0)
 		return;
 	
 	// PCM in format 44100:16:1 (for mono visualization) and 44100:16:2 (for stereo visualization) is supported
-	int16_t buf[itsSamples];
-	ssize_t data = read(itsFifo, buf, sizeof(buf));
+	int16_t buf[m_samples];
+	ssize_t data = read(m_fifo, buf, sizeof(buf));
 	if (data < 0) // no data available in fifo
 		return;
 	
-	if (itsOutputID != -1 && Global::Timer.tv_sec > itsTimer.tv_sec+Config.visualizer_sync_interval)
+	if (m_output_id != -1 && Global::Timer - m_timer > Config.visualizer_sync_interval)
 	{
-		Mpd.DisableOutput(itsOutputID);
+		Mpd.DisableOutput(m_output_id);
 		usleep(50000);
-		Mpd.EnableOutput(itsOutputID);
-		gettimeofday(&itsTimer, 0);
+		Mpd.EnableOutput(m_output_id);
+		m_timer = Global::Timer;
 	}
 	
 	void (Visualizer::*draw)(int16_t *, ssize_t, size_t, size_t);
@@ -131,9 +114,19 @@ void Visualizer::Update()
 	else
 #	endif // HAVE_FFTW3_H
 		draw = &Visualizer::DrawSoundWave;
-	
-	w->Clear();
-	ssize_t samples_read = data/sizeof(int16_t);
+
+	const ssize_t samples_read = data/sizeof(int16_t);
+	std::for_each(buf, buf+samples_read, [](int16_t &sample) {
+		int32_t tmp = sample * Config.visualizer_sample_multiplier;
+		if (tmp < std::numeric_limits<int16_t>::min())
+			sample = std::numeric_limits<int16_t>::min();
+		else if (tmp > std::numeric_limits<int16_t>::max())
+			sample = std::numeric_limits<int16_t>::max();
+		else
+			sample = tmp;
+	});
+
+	w.clear();
 	if (Config.visualizer_in_stereo)
 	{
 		int16_t buf_left[samples_read/2], buf_right[samples_read/2];
@@ -148,23 +141,33 @@ void Visualizer::Update()
 	}
 	else
 		(this->*draw)(buf, samples_read, 0, MainHeight);
-	w->Refresh();
+	w.refresh();
 }
 
-void Visualizer::SpacePressed()
+int Visualizer::windowTimeout()
+{
+	if (m_fifo >= 0 && Status::State::player() == MPD::psPlay)
+		return 1000/fps;
+	else
+		return Screen<WindowType>::windowTimeout();
+}
+
+void Visualizer::spacePressed()
 {
 #	ifdef HAVE_FFTW3_H
 	Config.visualizer_use_wave = !Config.visualizer_use_wave;
-	ShowMessage("Visualization type: %s", Config.visualizer_use_wave ? "Sound wave" : "Frequency spectrum");
+	Statusbar::printf("Visualization type: %1%",
+		Config.visualizer_use_wave ? "sound wave" : "frequency spectrum"
+	);
 #	endif // HAVE_FFTW3_H
 }
 
 void Visualizer::DrawSoundWave(int16_t *buf, ssize_t samples, size_t y_offset, size_t height)
 {
-	const int samples_per_col = samples/w->GetWidth();
+	const int samples_per_col = samples/w.getWidth();
 	const int half_height = height/2;
 	double prev_point_pos = 0;
-	const size_t win_width = w->GetWidth();
+	const size_t win_width = w.getWidth();
 	for (size_t i = 0; i < win_width; ++i)
 	{
 		double point_pos = 0;
@@ -173,15 +176,15 @@ void Visualizer::DrawSoundWave(int16_t *buf, ssize_t samples, size_t y_offset, s
 		point_pos /= samples_per_col;
 		point_pos /= std::numeric_limits<int16_t>::max();
 		point_pos *= half_height;
-		*w << XY(i, y_offset+half_height+point_pos) << Config.visualizer_chars[0];
-		if (i && abs(prev_point_pos-point_pos) > 2)
+		w << NC::XY(i, y_offset+half_height+point_pos) << Config.visualizer_chars[0];
+		if (i && fabs(prev_point_pos-point_pos) > 2)
 		{
 			// if gap is too big. intermediate values are needed
 			// since without them all we see are blinking points
 			const int breakpoint = std::max(prev_point_pos, point_pos);
 			const int half = (prev_point_pos+point_pos)/2;
 			for (int k = std::min(prev_point_pos, point_pos)+1; k < breakpoint; k += 2)
-				*w << XY(i-(k < half), y_offset+half_height+k) << Config.visualizer_chars[0];
+				w << NC::XY(i-(k < half), y_offset+half_height+k) << Config.visualizer_chars[0];
 		}
 		prev_point_pos = point_pos;
 	}
@@ -190,59 +193,69 @@ void Visualizer::DrawSoundWave(int16_t *buf, ssize_t samples, size_t y_offset, s
 #ifdef HAVE_FFTW3_H
 void Visualizer::DrawFrequencySpectrum(int16_t *buf, ssize_t samples, size_t y_offset, size_t height)
 {
-	for (unsigned i = 0, j = 0; i < itsSamples; ++i)
+	for (unsigned i = 0, j = 0; i < m_samples; ++i)
 	{
 		if (j < samples)
-			itsInput[i] = buf[j++];
+			m_fftw_input[i] = buf[j++];
 		else
-			itsInput[i] = 0;
+			m_fftw_input[i] = 0;
 	}
 	
-	fftw_execute(itsPlan);
+	fftw_execute(m_fftw_plan);
 	
 	// count magnitude of each frequency and scale it to fit the screen
-	for (unsigned i = 0; i < itsFFTResults; ++i)
-		itsFreqsMagnitude[i] = sqrt(itsOutput[i][0]*itsOutput[i][0] + itsOutput[i][1]*itsOutput[i][1])/1e5*height/5;
+	for (unsigned i = 0; i < m_fftw_results; ++i)
+		m_freq_magnitudes[i] = sqrt(m_fftw_output[i][0]*m_fftw_output[i][0] + m_fftw_output[i][1]*m_fftw_output[i][1])/2e4*height;
 	
-	const size_t win_width = w->GetWidth();
-	const int freqs_per_col = itsFFTResults/win_width /* cut bandwidth a little to achieve better look */ * 7/10;
+	const size_t win_width = w.getWidth();
+	// cut bandwidth a little to achieve better look
+	const int freqs_per_col = m_fftw_results/win_width * 7/10;
+	double bar_height;
+	size_t bar_real_height;
 	for (size_t i = 0; i < win_width; ++i)
 	{
-		size_t bar_height = 0;
+		bar_height = 0;
 		for (int j = 0; j < freqs_per_col; ++j)
-			bar_height += itsFreqsMagnitude[i*freqs_per_col+j];
-		bar_height = std::min(bar_height/freqs_per_col, height);
-		const size_t start_y = y_offset > 0 ? y_offset : height-bar_height;
-		const size_t stop_y = std::min(bar_height+start_y, w->GetHeight());
+			bar_height += m_freq_magnitudes[i*freqs_per_col+j];
+		// buff higher frequencies
+		bar_height *= log2(2 + i);
+		// moderately normalize the heights
+		bar_height = pow(bar_height, 0.5);
+		bar_real_height = std::min(size_t(bar_height/freqs_per_col), height);
+		const size_t start_y = y_offset > 0 ? y_offset : height-bar_real_height;
+		const size_t stop_y = std::min(bar_real_height+start_y, w.getHeight());
 		for (size_t j = start_y; j < stop_y; ++j)
-			*w << XY(i, j) << Config.visualizer_chars[1];
+			w << NC::XY(i, j) << Config.visualizer_chars[1];
 	}
 }
 #endif // HAVE_FFTW3_H
 
 void Visualizer::SetFD()
 {
-	if (itsFifo < 0 && (itsFifo = open(Config.visualizer_fifo_path.c_str(), O_RDONLY | O_NONBLOCK)) < 0)
-		ShowMessage("Couldn't open \"%s\" for reading PCM data: %s", Config.visualizer_fifo_path.c_str(), strerror(errno));
+	if (m_fifo < 0 && (m_fifo = open(Config.visualizer_fifo_path.c_str(), O_RDONLY | O_NONBLOCK)) < 0)
+		Statusbar::printf("Couldn't open \"%1%\" for reading PCM data: %2%",
+			Config.visualizer_fifo_path, strerror(errno)
+		);
 }
 
 void Visualizer::ResetFD()
 {
-	itsFifo = -1;
+	m_fifo = -1;
 }
 
 void Visualizer::FindOutputID()
 {
-	itsOutputID = -1;
+	m_output_id = -1;
 	if (!Config.visualizer_output_name.empty())
 	{
-		MPD::OutputList outputs;
-		Mpd.GetOutputs(outputs);
-		for (unsigned i = 0; i < outputs.size(); ++i)
-			if (outputs[i].first == Config.visualizer_output_name)
-				itsOutputID = i;
-		if (itsOutputID == -1)
-			ShowMessage("There is no output named \"%s\"!", Config.visualizer_output_name.c_str());
+		size_t idx = 0;
+		Mpd.GetOutputs([this, &idx](MPD::Output output) {
+			if (output.name() == Config.visualizer_output_name)
+				m_output_id = idx;
+			++idx;
+		});
+		if (m_output_id == -1)
+			Statusbar::printf("There is no output named \"%s\"", Config.visualizer_output_name);
 	}
 }
 
